local global = require "eatglobal_v1_3"
local utils = global.utils

local eat1963_cargocalculatedweight_config = {
	env = {
		debug = false,
	},
	userConfig = {
	},
	modifierSequenceList = {
		["01"] = "all_cargo_at_all_vehicle",
		["02"] = "capacities_to_compartments",
		["03"] = "cargo_calculated_weight",
	},
	cargodata = {
		initialized = false,
		calcData = {
			-- Einstellungen
			calcUnknownVehicle = true,
			-- Faktoren
			defaultVolumeFactor = 1.8,
			defaultWeightFactor = 1.2,
			-- sonstiges
			cargoFactor = 0.1518987342,
			defaultCapacity = 32,
			defaultTpfWeight = 1200,
			-- ab Version 1.7
			tpfDivFourFactor = 4,
			calcLoadSpeedFaktor = 10,
		},
		weights = {
		},
		cargoCompactness = {
			["PASSENGERS"] = 85,
			-- Baumstämme
			["LOGS"] = 800,
			-- Vieh
			["LIVESTOCK"] = 1000,
			-- Kohle
			["COAL"] = 1350,
			-- Eisenerz
			["IRON_ORE"] = 2400,
			-- Steine
			["STONE"] = 1400,
			-- Getreide
			["GRAIN"] = 600,
			-- Rohöl
			["CRUDE"] = 861,
			-- Stahl
			["STEEL"] = 7900,
			-- Bretter
			["PLANKS"] = 528,
			-- Kunststoff
			["PLASTIC"] = 1250, -- thx Baubus; Mittelwert aus "http://www.kern.de/index.php/de/kunststofflexikon/spezifisches-gewicht"
			-- Schlacke
			["SLAG"] = 1700, -- thx Baubus
			-- Leichtöl
			["OIL"] = 855,
			-- Baumaterial
			["CONSTRUCTION_MATERIALS"] = 3000,
			-- Maschinen
			["MACHINES"] = 3950,
			-- Kraftstoff
			["FUEL"] = 830,
			-- Werkzeuge
			["TOOLS"] = 2950,
			-- Lebensmittel
			["FOOD"] = 900, -- thx Baubus
			-- Güter
			["GOODS"] = 2500,
			--[[
					ATT 2.x Güter
			]]
			-- Ausstattung
			["EQUIPMENT"] = 2500,
			-- Bretter
			["PLANK"] = 528,
			--Fahrzeuge
			["VEHICLE"] = 150,
			-- Farben
			["COLOR"] = 1500,
			-- Gas
			["GAS"] = 540,
			-- Glas
			["GLASS"] = 1900, -- thx Baubus
			-- Gülle
			["SEWAGE"] = 1030,
			-- Koks
			["COKE"] = 560,
			-- Leder
			["LEATHER"] = 590,
			-- Möbel
			["FURNITURE"] = 349,
			-- Roheisen
			["IRON_PIG"] = 7874,
			-- Eisenspäne
			["SAWDUST"] = 2000,
			-- Sand
			["SAND"] = 1530,
			-- Schmierstoffe
			["LUBRICANTS"] = 900,
			-- Gemüse
			["VEGETABLE"] = 750, -- thx Baubus
			--[[
					Neue Industrie Güter 2.x
			]]
			-- Müll
			["WASTE"] = 150,
			-- Milch
			["MILK"] = 1030,
			-- Baumwolle
			["COTTON"] = 1510,
			-- KLeidung
			["CLOTHES"] = 1350,
			-- Düngemittel
			["FERTILIZERS"] = 950,
			-- Chemikalien
			["CHEMICAL"] = 950,
			-- Papier
			["PAPER"] = 800,
			-- Sägespäne
			["FILINGS"] = 220,
			-- Zement
			["CEMENT"] = 1900,
			-- Kalkstein
			["LIMESTONE"] = 1400,
			-- Mineralien
			["MINERALS"] = 1400,
			-- Uran
			["URAN"] = 19160,
			--[[
					Post-Mod von @nown
			]]
			-- sortierte Post
			["MAIL"] = 600,
			-- unsortierte Post
			["UNSORTED_MAIL"] = 600,
		},
		vehicleData = {
			-- Verbandswagen
			{
				list = { "eat1963_verbandswagen_02", "verbandswagen_v2.mdl", },
				data = { maxVolume = 45.7, maxWeight = 15, length = 9.6, },
			},
			-- Güterwagen Bauform Dresden gl und glhs
			{
				list = { "gl_dresden", "glhs_dresden", },
				data = { maxVolume = 57.6, maxWeight = 15, length = 12.1, },
			},
			-- Güterwagen  Rilns 654 (Modell von Jansch und ?)
			{
				list = { "rilns_654", "rilns654", },
				data = { maxVolume = 107.0, maxWeight = 65.0, length = 20.0, },
			},
			{
				list = {
					"40_tons.mdl",
				},
				data = { maxVolume = 91.233, maxWeight = 24, },
			},
			{
				list = { "man_19_304_1970.mdl", },
				data = { maxVolume = 53.219, maxWeight = 14, },
			},
			--- Volvo F12 Trucks von Onionjack
			-- Trailer
			{
				list = {
					"volvo_f12_box_v2.mdl",
					"volvo_f12_flatbed_v2.mdl",
					"volvo_f12_grain.mdl",
					"volvo_f12_kipper.mdl",
					"volvo_f12_livestock.mdl",
					"volvo_f12_logs.mdl",
					"volvo_f12_tanker.mdl",
				},
				data = { maxVolume = 90.0, maxWeight = 32, },
			},
			-- no Trailer
			{
				list = {
					"volvo_f12_box_notrailer_v2.mdl",
					"volvo_f12_flatbed_notrailer_v2.mdl",
					"volvo_f12_grain_notrailer.mdl",
					"volvo_f12_kipper_notrailer.mdl",
					"volvo_f12_livestock_notrailer.mdl",
					"volvo_f12_logs_notrailer.mdl",
					"volvo_f12_tanker_notrailer.mdl",
				},
				data = { maxVolume = 40.0, maxWeight = 15, },
			},
			-- WK50 von Spacky
			{
				list = {
					"W50_6x6_NVA.mdl",
					"W50_Radeberger.mdl",
					"W50_Minol.mdl",
				},
				data = { maxVolume = 14.0, maxWeight = 4,09, },
			},
		},
		vehicle = {
			-- geschlossene Güterwagen
			["res/models/model/vehicle/waggon/goods_1850_v2.mdl"] = {maxWeight = 8.0, maxVolume = 34.5},
			["res/models/model/vehicle/waggon/hbi1_v2.mdl"] = {maxWeight = 30.0, maxVolume = 105.0},
			["res/models/model/vehicle/waggon/usa/boxcar_1850.mdl"] = {maxWeight = 18.0, maxVolume = 55.0},
			["res/models/model/vehicle/waggon/usa/boxcar_1902.mdl"] = {maxWeight = 45.0, maxVolume = 100.0},
			["res/models/model/vehicle/waggon/usa/boxcar_1945.mdl"] = {maxWeight = 90.0, maxVolume = 130.0},
			
			-- Kesselwagen
			["res/models/model/vehicle/waggon/kesselwagen_1860.mdl"] = {maxWeight =  8.0,  maxVolume = 16.0},
			["res/models/model/vehicle/waggon/kesselwagen_1910.mdl"] = {maxWeight = 15.0, maxVolume = 25.0},
			["res/models/model/vehicle/waggon/kesselwagen_1950.mdl"] = {maxWeight = 55.0, maxVolume = 83.0},
			["res/models/model/vehicle/waggon/usa/tankcar_1860.mdl"] = {maxWeight = 18.0,  maxVolume = 30.0},
			["res/models/model/vehicle/waggon/usa/tankcar_1899.mdl"] = {maxWeight = 45.0, maxVolume = 50.0},
			["res/models/model/vehicle/waggon/usa/tankcar_1948.mdl"] = {maxWeight = 90.0, maxVolume = 103.0},
			
			--Rungenwagen
			["res/models/model/vehicle/waggon/rungenwagen_1850.mdl"] = {maxWeight =  8.0,  maxVolume = 13.0},
			["res/models/model/vehicle/waggon/rungenwagen_1890.mdl"] = {maxWeight = 15.0, maxVolume = 27.0},
			["res/models/model/vehicle/waggon/rungenwagen_1950.mdl"] = {maxWeight = 55.0, maxVolume = 60.0},
			["res/models/model/vehicle/waggon/usa/stake_car_1850.mdl"] = {maxWeight = 18.0,  maxVolume = 25.0},
			["res/models/model/vehicle/waggon/usa/stake_car_1895.mdl"] = {maxWeight = 45.0, maxVolume = 23.0},
			["res/models/model/vehicle/waggon/usa/stake_car_1952.mdl"] = {maxWeight = 90.0, maxVolume = 55.0},
			
			-- Schüttgutwagen
			["res/models/model/vehicle/waggon/open_1850.mdl"] = {maxWeight =  8.0,  maxVolume = 21.0},
			["res/models/model/vehicle/waggon/open_1910.mdl"] = {maxWeight = 15.0, maxVolume = 35.0},
			["res/models/model/vehicle/waggon/open_1975.mdl"] = {maxWeight = 55.0, maxVolume = 70.0},
			["res/models/model/vehicle/waggon/usa/gondola_1850.mdl"] = {maxWeight = 18.0,  maxVolume = 24.0},
			["res/models/model/vehicle/waggon/usa/gondola_1907.mdl"] = {maxWeight = 45.0, maxVolume = 45.0},
			["res/models/model/vehicle/waggon/usa/gondola_1955.mdl"] = {maxWeight = 90.0, maxVolume = 99.0},

			-- Lkw
			["res/models/model/vehicle/truck/horsewagon_1850.mdl"] = {maxWeight = 2.0, maxVolume = 2.5},
			["res/models/model/vehicle/truck/horse_cart.mdl"] = {maxWeight = 2.0, maxVolume = 3.0},
			["res/models/model/vehicle/truck/dmg_cannstatt.mdl"] = {maxWeight = 2.0, maxVolume = 5.0},
			["res/models/model/vehicle/truck/benz1912_lkw.mdl"] = {maxWeight = 3.0, maxVolume = 14.0},
			["res/models/model/vehicle/truck/opel_blitz_1930.mdl"] = {maxWeight = 3.5, maxVolume = 14.0},
			["res/models/model/vehicle/truck/man_19_304_1970.mdl"] = {maxWeight = 16.0, maxVolume = 42.0},
			["res/models/model/vehicle/truck/40_tons.mdl"] = {maxWeight = 30.0, maxVolume = 88.0},
			["res/models/model/vehicle/truck/usa/horsewagon_1850_usa.mdl"] = {maxWeight = 2.0, maxVolume = 2.5},
			["res/models/model/vehicle/truck/usa/american_horse_cart.mdl"] = {maxWeight = 2.0, maxVolume = 3.0},
			["res/models/model/vehicle/truck/usa/steam_lorry.mdl"] = {maxWeight = 2.0, maxVolume = 5.0, power = 10.0},
			["res/models/model/vehicle/truck/usa/mack_ac.mdl"] = {maxWeight = 2.0, maxVolume = 20.0},
			["res/models/model/vehicle/truck/usa/ford_m77.mdl"] = {maxWeight = 2.0, maxVolume = 20.0},
			["res/models/model/vehicle/truck/usa/studebaker_us6_u3.mdl"] = {maxWeight = 3.0, maxVolume = 40.0},
			["res/models/model/vehicle/truck/usa/studebaker_us6_u5.mdl"] = {maxWeight = 3.0, maxVolume = 40.0},
			["res/models/model/vehicle/truck/usa/studebaker_us6_u10.mdl"] = {maxWeight = 3.0, maxVolume = 40.0},
			["res/models/model/vehicle/truck/usa/american_truck_359.mdl"] = {maxWeight = 25.0, maxVolume = 88.0},
			["res/models/model/vehicle/truck/usa/cascadia_2009.mdl"] = {maxWeight = 35.0, maxVolume = 88.0},
			
			-- Schiffe
			["res/models/model/vehicle/ship/zoroaster.mdl"] = {maxWeight = 800.0, maxVolume = 1200.0},
			["res/models/model/vehicle/ship/vandal.mdl"] = {maxWeight = 1500.0, maxVolume = 1600.0},
			["res/models/model/vehicle/ship/viola.mdl"] = {maxWeight = 3000.0, maxVolume = 2500.0},
			["res/models/model/vehicle/ship/wilhelm.mdl"] = {maxWeight = 100.0, maxVolume = 100.0},
			["res/models/model/vehicle/ship/wilhelm_v2.mdl"] = {maxWeight = 100.0, maxVolume = 100.0},
			["res/models/model/vehicle/ship/klondike.mdl"] = {maxWeight = 1000.0, maxVolume = 1000.0},
			["res/models/model/vehicle/ship/klondike_v2.mdl"] = {maxWeight = 1000.0, maxVolume = 1000.0},
			["res/models/model/vehicle/ship/gms_axalp.mdl"] = {maxWeight = 2000.0, maxVolume = 1200.0},
			["res/models/model/vehicle/ship/gms_axalp_v2.mdl"] = {maxWeight = 2000.0, maxVolume = 1200.0},
			["res/models/model/vehicle/ship/merlin.mdl"] = {maxWeight = 3000.0, maxVolume = 3200.0},
			["res/models/model/vehicle/ship/merlin_v2.mdl"] = {maxWeight = 3000.0, maxVolume = 3200.0},
		},
		modData = {
			{
				-- 40 Tonner
				list = { "Nando_DE_Truck_Set1_1", },
				data = { maxVolume = 91.233, maxWeight = 24, },
			},
		},
		locked = {
			cargoTypes = {
				--"PASSENGERS",
			},
			carrierTypes = {
				--"AIR",
			},
			models = {
				"eat1963_candpbooster",
				"volvo_f12_box.mdl",
				"volvo_f12_box_notrailer.mdl",
				"volvo_f12_flatbed.mdl",
				"volvo_f12_flatbed_notrailer.mdl",
			},
		},
	},
}

local function getIfExistsVehicleData (transportVehicle)
	if transportVehicle.maxWeight and transportVehicle.maxVolume then
		return {
			maxWeight = transportVehicle.maxWeight,
			maxVolume = transportVehicle.maxVolume,
		}
	else
		return nil
	end
end

local function tryCalcVolume (config, transportVehicle)
	local maxCapacity = config.cargodata.calcData.defaultCapacity
	-- 07.09.2017 compartments hinzugefügt
	if transportVehicle.capacities then
		for i = 1, #transportVehicle.capacities do
			maxCapacity = math.max (maxCapacity, transportVehicle.capacities[i].capacity)
		end
	elseif transportVehicle.compartments then
		for i = 1, #transportVehicle.compartments do
			for j = 1, #transportVehicle.compartments[i] do
				for k = 1, #transportVehicle.compartments[i][j] do
					maxCapacity = math.max (maxCapacity, transportVehicle.compartments[i][j][k].capacity)
				end
			end
		end
	end
	--
	
	return {
		maxWeight = maxCapacity * config.cargodata.calcData.defaultWeightFactor,
		maxVolume = maxCapacity * config.cargodata.calcData.defaultVolumeFactor,
		calculated = true,
	}
end

local function findVehicleData (config, fileName)
	local function searchData ( data )
		for i = 1, #data do
			if utils.getIndex(data[i].list, fileName, function (params1, params2) return string.find (params2, params1, 1, true) end) > 0 then
				local result = data[i].data
				result.calculated = false
				return  result
			end
		end
		return nil
	end
	
	local result = searchData (config.cargodata.modData)
	if result == nil then
		result = searchData (config.cargodata.vehicleData)
	end
	
	return result
end

function eat1963_cargocalculatedweight_config.calcWeightAndVolume (config, fileName, transportVehicle)
	local result = getIfExistsVehicleData (transportVehicle) or config.cargodata.vehicle[fileName] or	findVehicleData(config, fileName) or	tryCalcVolume (config, transportVehicle)
	if result then
		result.isLocked = config.isLocked (config, fileName, transportVehicle)
	end
	return result
end

function eat1963_cargocalculatedweight_config.isAllowedCargoType (config, cargoType)
end

function eat1963_cargocalculatedweight_config.isLocked (config, fileName, transportVehicle)
	-- carrierTypes
	for i = 1, #config.cargodata.locked.carrierTypes do
		if transportVehicle.carrier == config.cargodata.locked.carrierTypes[i] then
			return true
		end
	end
	
	-- models
	return utils.getIndex(config.cargodata.locked.models, fileName, function (params1, params2) return string.find (params2, params1, 1, true) end) > 0
end

return eat1963_cargocalculatedweight_config